program Fux_first_species ! by Gilberto Agostinho (26.01.2014)
implicit none

integer, dimension(:), allocatable :: CF, CP ! CF = cantus firmus voice, CP = counterpoint voice
integer :: N ! number of notes in the cantus firmus
integer :: interval, next_interval
integer, dimension(12) :: attempt_values_up=(/1,-1,0,2,-2,3,-3,4,-4,5,7,-7/) ! possible leaps, prefereble up
integer, dimension(12) :: attempt_values_down=(/-1,1,0,-2,2,-3,3,-4,4,5,-7,7/) ! possible leaps, prefereble down
character (LEN=3), dimension(-2:13) :: scale=(/"g  ","a  ","b  ","c' ","d' ","e' ","f' ","g' ","a' ","b' ", &
"c''","d''","e''","f''","g''","a''"/) ! diatonic scale (white keys)
integer :: i, attempt, next_attempt_value ! auxiliary variables

open(unit=10,file="output.TXT")

100 write(*,*) "Please enter the number of notes in the cantus firmus"
read(*,*) N

allocate(CF(1:N),CP(1:N))

write(*,*) "Use the following notation of diatonic values to write the cantus firmus:"
write(*,*) "c' = 1, d' = 2, ... a'' = 13"
write(*,*) "b = 0, a = -1, g = -2"
write(*,*) "(Range = -2 until 13)"
write(*,*) 
do i=1,N
  write(*,"(1x,A40,1x,I2)") "Please enter the cantus firmus' note no.", i
  read(*,*) CF(i)
enddo

if (CF(1) /= CF(N)) then
  write(*,*) "The first and the last notes of the cantus firmus are not the same. Please try again"
  write(*,*)
  deallocate(CF,CP)
  goto 100
endif

if ( ABS(CF(N-1)-CF(N)) /= 1) then
  write(*,*) "The note before the last is not a second above or below the last note (Fux style). Please try again"
  write(*,*)
  deallocate(CF,CP)
  goto 100
endif

i = N
next_interval = 0
attempt = 1
next_attempt_value = 0

200 do while (i > 0) ! i starts at N and finishes at 0; i.e., the counterpoint is being created backwards! (there are advantages for this decision)
 
  if (i == N) then ! last note
      CP(i) = CF(i) + 7 ! last note is an octave up...
      if (CP(i) > 13) CP(i) = CF(i) - 7 ! ...or octave down, depending on the range
      i = i - 1
      goto 200
  endif

  if (i == (N-1)) then  ! the note before the last...
      if (CF(N) > CF (N-1) ) then  !...  must be approached by motion of second
        CP(i) = CP(i+1) + 1
        else
          CP(i) = CP(i+1) - 1
      endif
      next_interval = MOD ( ABS(CP(i)-CF(i)), 7)  ! interval between CP and CF
      i = i - 1
      goto 200
  endif
  
  if ( (i < (N-1)) .AND. (i > (2)) ) then ! i.e., all notes excluding the 1st, the 2nd, the one before the last and the last ones
    300 interval = ABS( CF(i) - CF(i+1) ) ! interval between counterpoint notes
    if (CF(i) > CF(i+1)) then ! preference to contrary motion if there is a skip between CP notes, else preference to parallel motion
      if (interval > 1) then
        CP(i) = CP(i+1) + attempt_values_down(attempt)
        else
          CP(i) = CP(i+1) + attempt_values_up(attempt)
      endif
      else
        if (interval > 1) then
          CP(i) = CP(i+1) + attempt_values_up(attempt)
          else
            CP(i) = CP(i+1) + attempt_values_down(attempt)
        endif
    endif
    if (( CP(i) < -2 ) .OR. (CP(i) > 13)) then ! if out of range, then skip
      attempt = attempt + 1
      goto 300
    endif
    if ( (i /= (N-1)) .AND. ( CP(i) == CP(i+1) ) .AND. ( CP(i) == CP(i+2) )) then ! only two pitch repetitions allowed (aaa)
      attempt = attempt + 1
      goto 300
    endif    
    if (( CP(i) == CP(i+1) ) .AND. (CP(i+2) == CP(i+3))) then ! no pitch repetitions followed by each other (aabb)
      attempt = attempt + 1
      goto 300
    endif 
    if ( ((CP(i) == 4) .OR. (CP(i) == 11)) .AND. &
    ((CP(i+1) == 0) .OR. (CP(i+1) == 7) .OR. (CF(i+1) == 0) .OR. (CF(i+1) == 7)) ) then ! no tritones
      attempt = attempt + 1
      goto 300
    endif
    if ( ((CP(i) == 0) .OR. (CP(i) == 7)) .AND. &
    ((CP(i+1) == 4) .OR. (CP(i+1) == 11) .OR. (CF(i+1) == 4) .OR. (CF(i+1) == 11)) ) then ! no tritones
      attempt = attempt + 1
      goto 300
    endif
    if (attempt > 13) then ! if all attempts failed, the previously decided note has to be changed
      attempt = next_attempt_value + 1
      i = i + 1
      goto 200
    endif
    interval = ABS( CP(i)-CF(i) ) ! interval between CP and CF
    if (interval > 9) then ! no intervals larger than a 10th
      attempt = attempt + 1 ! then try another possibility
      goto 300
    endif
    interval = MOD( interval, 7 ) ! interval withing an octave (i.e., 9th->2nd, 10th-> 3rd, etc...
    if ( ((MOD(CP(i),7) == 0).AND.(MOD(CF(i),7) == 4)) .OR. ((MOD(CP(i),7) == 4).AND.(MOD(CF(i),7) == 0)) ) then ! if interval is a diminished fifth
      attempt = attempt + 1 ! if not ALL of those if's are true, this line is executed
      goto 300
    endif
    if ((interval == 0) .OR. (interval == 2) .OR. (interval == 4) .OR. (interval == 5)) then ! if consonant
      if ( .NOT. (((interval == 0) .OR. (interval == 4)) .AND. (next_interval == interval )) ) then ! if NOT parallel 5a or 8a
        if ((next_interval == 0) .OR. (next_interval == 4)) then! if next interval is an 5a or an 8a
          if ( ((CF(i+1) >= CF(i)) .AND. (CP(i+1) <= CP(i))) .OR. ((CF(i+1) <= CF(i)) .AND. (CP(i+1) >= CP(i))) ) then ! if NO hidden 5as or octaves
            i = i - 1
            next_interval = interval
            next_attempt_value = attempt
            attempt = 1
            goto 200
          endif
          else ! if next interval is not an 5a or 8a
            i = i - 1
            next_interval = interval
            next_attempt_value = attempt
            attempt = 1
            goto 200
        endif
      endif
    endif
    attempt = attempt + 1 ! if not ALL of those if's are true, this line is executed
    goto 300
  endif
 
  if (i == 2) then ! second note
    400 interval = ABS( CF(i) - CF(i+1) ) ! interval between counterpoint notes
    if (CF(i) > CF(i+1)) then ! preference to contrary motion if there is a skip between CP notes, else preference to parallel motion
      if (interval > 1) then
        CP(i) = CP(i+1) + attempt_values_down(attempt)
        else
          CP(i) = CP(i+1) + attempt_values_up(attempt)
      endif
      else
        if (interval > 1) then
          CP(i) = CP(i+1) + attempt_values_up(attempt)
          else
            CP(i) = CP(i+1) + attempt_values_down(attempt)
        endif
    endif
    if (( CP(i) < -2 ) .OR. (CP(i) > 13)) then ! if out of range, then skip
      attempt = attempt + 1
      goto 400
    endif
    if (( CP(i) == CP(i+1) ) .AND. (CP(i) == CP(i+2))) then ! only two pitch repetitions allowed (aaa)
      attempt = attempt + 1
      goto 400
    endif    
    if (( CP(i) == CP(i+1) ) .AND. (CP(i+2) == CP(i+3))) then ! no pitch repetitions followed by each other (aabb)
      attempt = attempt + 1
      goto 400
    endif    
    if ( ((CP(i) == 4) .OR. (CP(i) == 11)) .AND. &
    ((CP(i+1) == 0) .OR. (CP(i+1) == 7) .OR. (CF(i+1) == 0) .OR. (CF(i+1) == 7)) ) then ! no tritones
      attempt = attempt + 1
      goto 400
    endif
    if ( ((CP(i) == 0) .OR. (CP(i) == 7)) .AND. &
    ((CP(i+1) == 4) .OR. (CP(i+1) == 11) .OR. (CF(i+1) == 4) .OR. (CF(i+1) == 11)) ) then ! no tritones
      attempt = attempt + 1
      goto 400
    endif
    if (attempt > 13) then ! if all attempts failed, the previously decided note has to be changed
      attempt = next_attempt_value + 1
      i = i + 1
      goto 200
    endif
    interval = ABS( CP(i)-CF(i) ) ! interval between CP and CF
    if (interval > 9) then ! no intervals larger than a 10th
      attempt = attempt + 1 ! then try another possibility
      goto 400
    endif
    interval = MOD( interval, 7 ) ! interval withing an octave (i.e., 9th->2nd, 10th-> 3rd, etc...
    if ( ((MOD(CP(i),7) == 0).AND.(MOD(CF(i),7) == 4)) .OR. ((MOD(CP(i),7) == 4).AND.(MOD(CF(i),7) == 0)) ) then ! if interval is a diminished fifth
      attempt = attempt + 1 ! if not ALL of those if's are true, this line is executed
      goto 400
    endif
    if ((interval == 0) .OR. (interval == 2) .OR. (interval == 4) .OR. (interval == 5)) then ! if consonant
      if ( .NOT. (((interval == 0) .OR. (interval == 4)) .AND. (next_interval == interval )) ) then ! if NOT parallel 5a or 8a
        if ((next_interval == 0) .OR. (next_interval == 4)) then! if next interval is an 5 or an 8a
          if ( ((CF(i+1) >= CF(i)) .AND. (CP(i+1) <= CP(i))) .OR. ((CF(i+1) <= CF(i)) .AND. (CP(i+1) >= CP(i))) ) then ! if NO hidden 5as or octaves
            i = i - 1
            next_interval = interval
            next_attempt_value = attempt
            attempt = 1
            goto 200
          endif
          else ! if next interval is not an 5a or 8a
            i = i - 1
            next_interval = interval
            next_attempt_value = attempt
            attempt = 1
            goto 200
        endif
      endif
    endif
    attempt = attempt + 1 ! if not ALL of those if's are true, this line is executed
    goto 400
  endif
   
  if (i == 1) then ! first note    
    500 interval = ABS( CF(i) - CF(i+1) ) ! interval between counterpoint notes
    if (CF(i) > CF(i+1)) then ! preference to contrary motion if there is a skip between CP notes, else preference to parallel motion
      if (interval > 1) then
        CP(i) = CP(i+1) + attempt_values_down(attempt)
        else
          CP(i) = CP(i+1) + attempt_values_up(attempt)
      endif
      else
        if (interval > 1) then
          CP(i) = CP(i+1) + attempt_values_up(attempt)
          else
            CP(i) = CP(i+1) + attempt_values_down(attempt)
        endif
    endif
    if (( CP(i) < -2 ) .OR. (CP(i) > 13)) then ! if out of range, then skip
      attempt = attempt + 1
      goto 500
    endif
    if (( CP(i) == CP(i+1) ) .AND. (CP(i) == CP(i+2))) then ! only two pitch repetitions allowed (aaa)
      attempt = attempt + 1
      goto 500
    endif    
    if (( CP(i) == CP(i+1) ) .AND. (CP(i+2) == CP(i+3))) then ! no pitch repetitions followed by each other (aabb)
      attempt = attempt + 1
      goto 500
    endif    
    if (attempt > 13) then ! if all attempts failed, the previously decided note has to be changed
      attempt = next_attempt_value + 1
      i = i + 1
      goto 500
    endif
    interval = ABS( CP(i)-CF(i) ) ! interval between CP and CF
    if (interval > 9) then ! no intervals larger than a 10th
      attempt = attempt + 1 ! then try another possibility
      goto 500
    endif
    interval = MOD( interval, 7 ) ! interval withing an octave (i.e., 9th->2nd, 10th-> 3rd, etc...
    if ((interval == 0) .AND. (next_interval /= interval )) then ! if 8a or unis. and next NOT an octave
      i = i - 1
      else if ( (interval == 4) .AND. ( CP(i) > CF(i) ) .AND. (next_interval /= interval ) ) then ! if 5th and next NOT 5th and CP above CF
        i = i - 1 
      else
        attempt = attempt + 1 ! if not ALL of those if's are true, this line is executed
        goto 500
    endif      
  endif

enddo

! ***************************************************************************************************************************************
! "Score"

write(10,"(A3,1x)",advance="NO") "CP:"
do i=1,N
  if (i /= (N-1)) then
    write(10,"(A3,2x)",advance="NO") scale(CP(i))
    else
      if ( ( ((MOD(CF(N),7)==2) .AND. ( MOD(CF(N-2),7) /= 5 )) .OR. ((MOD(CF(N),7)==5) .AND. ( MOD(CF(N-2),7) /= 1 )) &
      .OR. ((MOD(CF(N),7)==6) .AND. ( MOD(CF(N-2),7) /= 2 )) ) .AND. (CP(N-1) < CP(N)) ) then ! If modes are dorian, mixolydian or aeolian, then leading tone might have an accidental
        if ( ( ABS(CP(N-2) - CP(N-1)) == 1 ) .OR. ( ABS(CP(N-2) - CP(N-1)) == 2 ) .OR. ( ABS(CP(N-2) - CP(N-1)) == 5 ) ) then
          write(10,"(A3,A1,1x)",advance="NO") scale(CP(i)),"#"
          else
            write(10,"(A3,2x)",advance="NO") scale(CP(i))
        endif
        else
          write(10,"(A3,2x)",advance="NO") scale(CP(i))
      endif
  endif
enddo    
write(10,"(A2)") "||"

write(10,"(A3,1x)",advance="NO") "CF:"
do i=1,N
  if (i /= (N-1)) then
    write(10,"(A3,2x)",advance="NO") scale(CF(i))
    else
      if ( ( ((MOD(CF(N),7)==2) .AND. ( MOD(CP(N-2),7) /= 5 )) .OR. ((MOD(CF(N),7)==5) .AND. ( MOD(CP(N-2),7) /= 1 )) &
      .OR. ((MOD(CF(N),7)==6) .AND. ( MOD(CP(N-2),7) /= 2 )) ) .AND. (CF(N-1) < CF(N)) ) then ! If modes are dorian, mixolydian or aeolian, then leading tone might have an accidental
        if ( ( ABS(CF(N-2) - CF(N-1)) == 1 ) .OR. ( ABS(CF(N-2) - CF(N-1)) == 2 ) .OR. ( ABS(CF(N-2) - CF(N-1)) == 5 ) ) then
          write(10,"(A3,A1,1x)",advance="NO") scale(CF(i)),"#"
          else
            write(10,"(A3,2x)",advance="NO") scale(CF(i))
        endif
        else
          write(10,"(A3,2x)",advance="NO") scale(CF(i))
      endif
  endif
enddo  
write(10,"(A2)") "||" 

end program Fux_first_species